#define float double
#define SPP 256          /* SPATIAL POINTS IN THE PICTURE */
#define TPP 500          /* TIME POINTS IN THE PICTURE */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "nrutil.h"
#include "nrutil.c"
#include "complex.c"


fcomplex *complexvector(nl,nh)
int nl,nh;
{
        fcomplex *v;

        v=(fcomplex *)malloc((unsigned) (nh-nl+1)*sizeof(fcomplex));
        if (!v) nrerror("allocation failure in complexvector()");
        return v-nl;
}

void free_complexvector(v,nl,nh)
fcomplex *v;
int nl,nh;
{
        free((char*) (v+nl));
}


 
void ctridag(a,b,c,r,u,n)
fcomplex *a,*b,*c,*r,*u;
int n;
{
	int j;
	fcomplex bet,*gam,*complexvector();
	void nrerror(),free_complexvector();


	gam=complexvector(1,n);
	if ((b[1].r == 0.0) && (b[1].i == 0.0)) nrerror("Error 1 in TRIDAG");
	u[1]=Cdiv(r[1],(bet=b[1]));
	for (j=2;j<=n;j++) {
		gam[j]=Cdiv(c[j-1],bet);
		bet=Csub(b[j],Cmul(a[j],gam[j]));
		if ((bet.r == 0.0) && (bet.i == 0.0))	nrerror("Error 2 in TRIDAG");
		u[j]=Cdiv(Csub(r[j],Cmul(a[j],u[j-1])),bet);
	}
	for (j=(n-1);j>=1;j--)
		u[j] = Csub(u[j], Cmul(gam[j+1],u[j+1]));
	
	free_complexvector(gam,1,n);
}



void rhs(dz2, dt, potvec, psivec, rhsvec, J)
float dz2, dt, *potvec;
fcomplex *psivec, *rhsvec;
int J;
{
	int j;
	
	rhsvec[1]=Cadd(Cmul(psivec[1], 
			    Complex(2.0*dt*(1.0 + dz2*potvec[1]),
				    4.0*dz2 
			    	    )
			    ), 
		       Cmul(Complex(-1.0*dt,
		       		    0.0
		                    ), 
		            psivec[2]
		            )
		       );

	for (j=2;j<=J-2;j++)  
		rhsvec[j]=Cadd(Cmul(psivec[j], 
				    Complex(2.0*dt*(1.0 + dz2*potvec[j]),
					    4.0*dz2 
				    	    )
				    ), 
			       Cmul(Complex(-1.0*dt,
			       		    0.0
			                    ), 
			            Cadd(psivec[j+1], 
			                 psivec[j-1]
			                 )
			            )
			       );

	rhsvec[J-1]=Cadd(Cmul(psivec[J-1], 
			    Complex(2.0*dt*(1.0 + dz2*potvec[J-1]),
				    4.0*dz2 
			    	    )
			    ), 
		       Cmul(Complex(-1.0*dt,
		       		    0.0
		                    ), 
		            psivec[J-2]
		            )
		       );

}



main()
{
	float t, tmin, tmax, dt, zmin, zmax, dz, dz2, dt0, p0, z0, w0;
	float Pi, *zvec, *potvec, *psia2vec, *tvec;
	float *vector();
	fcomplex *psivec, *diagvec, *subvec, *supervec, *rhsvec;
	fcomplex *complexvector();
	int k, j, J, smpl, tc;
	void free_vector(), free_complexvector();


	Pi=4.0*atan(1.0);
	
	zmin = 0.0;
	zmax = 10.0;
	J = 1024;
	tmin = 0.0;
	tmax = 10.0;
	dt0 = 0.001;

	dz=(zmax-zmin)/((double) J);
	dz2=dz*dz;

	smpl=J/SPP;

	psivec=complexvector(1,J-1);
	diagvec=complexvector(1,J-1);
	rhsvec=complexvector(1,J-1);
	subvec=complexvector(1,J-1);
	supervec=complexvector(1,J-1);
	psia2vec=vector(1,J-1);
	zvec=vector(1,J-1);
	potvec=vector(1,J-1);
	tvec = vector(1,TPP);

	for(tc=1; tc<=TPP; tc++) tvec[tc] = tmin + ((double)tc)*(tmax-tmin)/TPP;


//	p0 = 15.0;
	p0 = 3.0;

	z0 = 3.0;
	w0 = 0.5;

	for(j=1;j<=J-1;j++) 
	{	
		zvec[j] = zmin + j*dz;
		
		if(zvec[j]<5.0 || zvec[j]>7.5) {potvec[j] = 0.0;} else {potvec[j] = 10.0;}
//		potvec[j] = 0.0;

		psivec[j] = Complex( exp(-(zvec[j]-z0)*(zvec[j]-z0)/(w0*w0)) * cos(p0*zvec[j]),
							 exp(-(zvec[j]-z0)*(zvec[j]-z0)/(w0*w0)) * sin(p0*zvec[j]) );
		psia2vec[j] = psivec[j].r * psivec[j].r + psivec[j].i * psivec[j].i;
	}
	



	printf("{\n");

	printf("{ 0.0,");
	for(j=smpl;j<J;j += smpl)
		printf("%lf, ", psia2vec[j]);
	printf("0.0 } \n");




	t  = tmin;
	dt = dt0;
	tc = 1;
	while (t < tmax)
	{	
		dt = dt0;

		if ((t<tvec[tc]) && (t+dt>=tvec[tc])) dt = tvec[tc] - t;

		t += dt;
			
		for (j=1;j<=J-1;j++)
		{	
			subvec[j].r = dt;
			subvec[j].i = 0.0;
			supervec[j].r = dt;
			supervec[j].i = 0.0;

			diagvec[j]=Complex(-2.0*dt*(1.0 + dz2*potvec[j]),
							4.0*dz2 );
		}
		
		rhs(dz2, dt, potvec, psivec, rhsvec, J);
			
		ctridag(subvec, diagvec, supervec, rhsvec, psivec, J-1);
			

		if(t == tvec[tc])
		{	
			printf(", { 0.0,");
			for(j=smpl;j<J;j += smpl)
			{		
				psia2vec[j] = psivec[j].r * psivec[j].r + psivec[j].i * psivec[j].i;
				printf("%lf, ", psia2vec[j]);
			}
			printf("0.0 } \n");
			
			tc++;
		}

	}

	printf("}\n");

	free_complexvector(psivec,1,J-1);
	free_complexvector(diagvec,1,J-1);
	free_complexvector(rhsvec,1,J-1);
	free_complexvector(subvec,1,J-1);
	free_complexvector(supervec,1,J-1);
	free_vector(psia2vec,1,J-1);
	free_vector(zvec,1,J-1);
	free_vector(potvec,1,J-1);

}

