#include "nrutil.h"
#include "nrutil.c"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

void tridag(double a[], double b[], double c[], double r[], double u[], unsigned long n)
/*
Solves for a vector u[1..n] the tridiagonal linear set given by equation (2.4.1). a[1..n],
b[1..n], c[1..n], and r[1..n] are input vectors and are not modified.
*/
{
	unsigned long j;
	double bet,*gam;
	gam=dvector(1,n); // One vector of workspace, gam is needed.
	
	if (b[1] == 0.0) nrerror("Error 1 in tridag");
					//If this happens then you should rewrite your equations as a set of order N - 1, w ith u2 trivially eliminated.

	u[1]=r[1]/(bet=b[1]);

	for (j=2;j<=n;j++) { //Decomposition and forward substitution.

		gam[j]=c[j-1]/bet;
		bet=b[j]-a[j]*gam[j];
		if (bet == 0.0) nrerror("Error 2 in tridag"); //Algorithm fails; see below.
		u[j]=(r[j]-a[j]*u[j-1])/bet;
	}
	
	for (j=(n-1);j>=1;j--)
		u[j] -= gam[j+1]*u[j+1]; //Backsubstitution.
	
	free_dvector(gam,1,n);
}



void main()
// hovezetes egyenletenek megoldasa Crank-Nicolson modszerrel
{
	double *a, *b, *c, *u, *r, **v, alfa, dt, dx, lambda, Pi;
	int j, n, J, N;
//beolv
dt = 0.001;

N = 100;

J = 1000;

dx = 1.0 / J;

// tarfoglalas

	a = dvector(1,J-1);
	b = dvector(1,J-1);
	c = dvector(1,J-1);
	u = dvector(1,J-1);
	r = dvector(1,J-1);
	v = dmatrix(0,N,0,J);

	lambda = 1.0;

	alfa = lambda * dt / (dx * dx);

// inicializalas

	Pi=4.0*atan(1.0);


	for(n=0;n<=N;n++)
	{						// Dirichlet-fele peremfeltetel
		v[n][0] = 0.0;		// a bal szelen,
		v[n][J] = 0.0;		// a jobb szelen,
	}						// lehetne idofuggo is!

	for(j=1;j<=J-1;j++)
	{
		a[j] = - alfa;
		b[j] = 2.0 + 2.0 * alfa;
		c[j] = - alfa;
//		if(j<J/4 || j>3*J/4) {v[0][j] = 0.0;} else {v[0][j] = 1.0;}
		v[0][j] =	
					//0.0; 
					exp(-(j*dx - 0.5)*(j*dx - 0.5)*500);
//					sin(Pi*j*dx);
	}

// idobeli lepegetes
	
	for(n=0;n<N;n++)
	{
		for(j=1;j<=J-1;j++)
		{
			r[j] = alfa * v[n][j+1] + (2.0 - 2.0 * alfa) * v[n][j] + alfa * v[n][j-1];

		}
		r[1] = r[1] + alfa * v[n+1][0];
		r[J-1] = r[J-1] + alfa * v[n+1][J];

		tridag(a,b,c,r,u,J-1);
		
		for(j=1;j<=J-1;j++)
		{
			v[n+1][j] = u[j];
		}
		
	}

//kiirat's


	printf("{\n");
	for(n=0;n<N;n++)
	{
		printf("{ ");
		for(j=0;j<J;j++)
		{
			printf("%.16f , ", v[n][j]);

		}
		printf("%.16f } ,\n", v[n][J]);
	}

	printf("{ ");
		for(j=0;j<J;j++)
		{
			printf("%.16f , ", v[N][j]);

		}
		printf("%.16f } \n }", v[N][J]);


// memoria felszabaditas

	free_dvector(a,1,J-1);
	free_dvector(b,1,J-1);
	free_dvector(c,1,J-1);
	free_dvector(u,1,J-1);
	free_dvector(r,1,J-1);
	free_dmatrix(v,0,N,0,J);

}