#include "nrutil.h"
#include "nrutil.c"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

void tridag(double a[], double b[], double c[], double r[], double u[], int n1, 
                        int n2)
/*
Solves for a vector u[n1..n2] the tridiagonal linear set given by equation (2.4.1). a[n1..n2],
b[n1..n2], c[n1..n2], and r[n1..n2] are input vectors and are not modified.
*/
{
        int j;
        double bet,*gam;
        gam=dvector(n1,n2); // One vector of workspace, gam is needed.
  
        if (b[n1] == 0.0) nrerror("Error 1 in tridag");
                                        //If this happens then you should rewrite your equations as a set of order N - 1, w ith u2 trivially eliminated.

        u[n1]=r[n1]/(bet=b[n1]);

        for (j=n1+1;j<=n2;j++) { //Decomposition and forward substitution.

                gam[j]=c[j-1]/bet;
                bet=b[j]-a[j]*gam[j];
                if (bet == 0.0) nrerror("Error 2 in tridag"); //Algorithm fails; see below.
                u[j]=(r[j]-a[j]*u[j-1])/bet;
        }
  
        for (j=(n2-1);j>=n1;j--)
                u[j] -= gam[j+1]*u[j+1]; //Backsubstitution.
  
        free_dvector(gam,n1,n2);
}



void main()
// hovezetes egyenletenek megoldasa Crank-Nicolson modszerrel
{
        double *a, *b, *c, *u, *r, **v, alfa, dt, dx, lambda, *nb1, *nb2;
        int j, n, J, N;
//beolv
dt = 0.001;

N = 100;

J = 1000;

dx = 1.0 / J;

// tarfoglalas

        a = dvector(0,J);
        b = dvector(0,J);
        c = dvector(0,J);
        u = dvector(0,J);
        r = dvector(0,J);
        v = dmatrix(0,N,0,J);
        nb1 = dvector(0,N);
        nb2 = dvector(0,N);

        lambda = 1.0;

        alfa = lambda * dt / (dx * dx);

// inicializalas

        for(n=0;n<=N;n++)
        {                                               // Neumann-fele peremfeltetel (a derivlt rtke a peremen)
                nb1[n] = 0.0;           // a bal szelen,
                nb2[n] = 0.0;           // a jobb szelen,
        }                                               // lehetne idofuggo is!

                                                       // A Neumann-fele peremfeltetelt 2-rendben pontosan vesszuk figyelembe.
        for(j=0;j<=J;j++)
        {
                a[j] = - alfa;
                b[j] = 2.0 + 2.0 * alfa;
                c[j] = - alfa;
        }
        c[0] = -2.0 * alfa;
        a[J] = -2.0 * alfa;

        for(j=0;j<=J;j++)
        {  
                //      if(j<J/4 || j>3*J/4) {v[0][j] = 0.0;} else {v[0][j] = 1.0;}
                v[0][j] =  
                                        //0.0; 
                                        exp(-(j*dx - 0.5)*(j*dx - 0.5)*50);
        }

// idobeli lepegetes
  
        for(n=0;n<N;n++)
        {
                for(j=1;j<=J-1;j++)
                {
                        r[j] = alfa * v[n][j+1] + (2.0 - 2.0 * alfa) * v[n][j] + alfa * v[n][j-1];

                }
                r[0] = -2.0 * alfa * dx * (nb1[n+1] + nb1[n]) + 2.0 * alfa * v[n][1] + (2.0 - 2.0 * alfa) * v[n][0];
                r[J] = 2.0 * alfa * dx * (nb2[n+1] + nb2[n]) + 2.0 * alfa * v[n][J-1] + (2.0 - 2.0 * alfa) * v[n][J];

                tridag(a,b,c,r,u,0,J);
  
                for(j=0;j<=J;j++)
                {
                        v[n+1][j] = u[j];
                }
  
        }

//kiirat's


        printf("{\n");
        for(n=0;n<N;n++)
        {
                printf("{ ");
                for(j=0;j<J;j++)
                {
                        printf("%.16f , ", v[n][j]);

                }
                printf("%.16f } ,\n", v[n][J]);
        }

        printf("{ ");
                for(j=0;j<J;j++)
                {
                        printf("%.16f , ", v[N][j]);

                }
                printf("%.16f } \n }", v[N][J]);


// memoria felszabaditas

        free_dvector(a,0,J);
        free_dvector(b,0,J);
        free_dvector(c,0,J);
        free_dvector(u,0,J);
        free_dvector(r,0,J);
        free_dmatrix(v,0,N,0,J);
        free_dvector(nb1,0,N);
        free_dvector(nb2,0,N);

}
